<?php
/**
 * CreepTD is an Open Source Online Multiplayer TowerDefense game for 1-4
 * players playing on one map. It's the mother of all TD games.
 *
 *                      ---   www.creeptd.com   ---
 *
 * CreepTD PHP library Copyright (C) 2011 by
 * - Daniel Wirtz, virtunity media (http://virtunity.com)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

/**
 * CreepTD API connector.
 * 
 * @author Daniel
 */
class CreepTD_API {
    private $base;
    private $handler;
    
    /**
     * Create an API instance.
     * @param string $base Base URI
     * @param object $handler Handler instance
     */
    public function __construct($base, $handler) {
        if ($base{strlen($base)-1} != '/') {
            $base .= "/";
        }
        $this->base = $base;
        $this->handler = $handler;
    }
    
    /**
     * Execute and dispatch to handler.
     */
    public function execute() {
        $command = substr($_SERVER['RERQUEST_URI'], strlen($this->base));
        if (($pos = strpos($command, "?")) !== false) {
            $command = substr($command, 0, $pos);
        }
        if (strlen($command) > 0 && preg_match($command, "/^\w+$/")) {
            $data = null;
            if ($_SERVER['HTTP_CONTENT_TYPE'] == "text/json" && strtoupper($_SERVER['HTTP_METHOD']) == "POST") {
                $raw = file_get_contents('php://input');
                if (strlen($raw) > 0) {
                    $data = json_decode($raw);
                }
            }
            if (method_exists($this->handler, $command)) {
                $result = $this->handler->__call($command, $data);
                $this->sendResult($result);
            } else {
                $this->sendError("nosuchcommand");
            }
        } else {
            $this->sendError("invalidrequest");
        }
    }
    
    /**
     * Send an error result.
     * @param string $message
     */
    private function sendError($message) {
        echo json_encode(array("error" => $message));
        die();
    }
    
    /**
     * Send a result.
     * @param mixed $result Result data
     */
    private function sendResult($result) {
        echo json_encode(array("result" => $result));
        die();
    }
}
?>